pub struct SaplingNoteEncryption<R: RngCore> { /* private fields */ }Expand description
An API for encrypting Sapling notes.
This struct provides a safe API for encrypting Sapling notes. In particular, it enforces that fresh ephemeral keys are used for every note, and that the ciphertexts are consistent with each other.
Implements section 4.17.1 of the Zcash Protocol Specification. NB: the example code is only covering the pre-Canopy case.
Examples
extern crate ff;
extern crate rand_core;
extern crate zcash_primitives;
use ff::Field;
use rand_core::OsRng;
use zcash_primitives::{
keys::{OutgoingViewingKey, prf_expand},
note_encryption::{Memo, SaplingNoteEncryption},
primitives::{Diversifier, PaymentAddress, Rseed, ValueCommitment},
};
let mut rng = OsRng;
let diversifier = Diversifier([0; 11]);
let pk_d = diversifier.g_d().unwrap();
let to = PaymentAddress::from_parts(diversifier, pk_d).unwrap();
let ovk = Some(OutgoingViewingKey([0; 32]));
let value = 1000;
let rcv = jubjub::Fr::random(&mut rng);
let cv = ValueCommitment {
value,
randomness: rcv.clone(),
};
let rcm = jubjub::Fr::random(&mut rng);
let note = to.create_note(value, Rseed::BeforeZip212(rcm)).unwrap();
let cmu = note.cmu();
let mut enc = SaplingNoteEncryption::new(ovk, note, to, Memo::default(), &mut rng);
let encCiphertext = enc.encrypt_note_plaintext();
let outCiphertext = enc.encrypt_outgoing_plaintext(&cv.commitment().into(), &cmu);Implementations
Sourceimpl<R: RngCore + CryptoRng> SaplingNoteEncryption<R>
impl<R: RngCore + CryptoRng> SaplingNoteEncryption<R>
Sourcepub fn new(
ovk: Option<OutgoingViewingKey>,
note: Note,
to: PaymentAddress,
memo: Memo,
rng: R,
) -> Self
pub fn new( ovk: Option<OutgoingViewingKey>, note: Note, to: PaymentAddress, memo: Memo, rng: R, ) -> Self
Creates a new encryption context for the given note.
Setting ovk to None represents the ovk = ⊥ case, where the note cannot be
recovered by the sender.
Sourceimpl<R: RngCore> SaplingNoteEncryption<R>
impl<R: RngCore> SaplingNoteEncryption<R>
Sourcepub fn epk(&self) -> &SubgroupPoint
pub fn epk(&self) -> &SubgroupPoint
Exposes the ephemeral public key being used to encrypt this note.
Sourcepub fn encrypt_note_plaintext(&self) -> [u8; 580]
pub fn encrypt_note_plaintext(&self) -> [u8; 580]
Generates encCiphertext for this note.
Sourcepub fn encrypt_outgoing_plaintext(
&mut self,
cv: &ExtendedPoint,
cmu: &Scalar,
) -> [u8; 80]
pub fn encrypt_outgoing_plaintext( &mut self, cv: &ExtendedPoint, cmu: &Scalar, ) -> [u8; 80]
Generates outCiphertext for this note.